@@ -27,11 +27,18 @@
 #include "openssl/ssl.h"
 #include "openssl/err.h"
 #include "String.hpp"
+#include "CertificateAuthority.hpp"
 #endif
 
 #ifdef __SSLMITM
 extern bool reloadconfig;
+
+#ifndef X509_V_FLAG_TRUSTED_FIRST
+#warning "X509_V_FLAG_TRUSTED_FIRST not available, chain creation will be unreliable"
+#define X509_V_FLAG_TRUSTED_FIRST 0
 #endif
+#endif
+
 // IMPLEMENTATION
 
 // constructor - create an INET socket & clear address structs
@@ -220,12 +227,13 @@ Socket *Socket::accept()
 
 #ifdef __SSLMITM
 //use this socket as an ssl client
-int Socket::startSslClient(const std::string &certificate_path)
+int Socket::startSslClient(const std::string &certificate_path, String hostname)
 {
     if (isssl) {
         stopSsl();
     }
 
+    ERR_clear_error();
     ctx = SSL_CTX_new(SSLv23_client_method());
     if (ctx == NULL) {
 //needed to get the errors when creating ctx
@@ -235,6 +243,7 @@ int Socket::startSslClient(const std::string &certificate_path)
         //syslog(LOG_ERR, "error creating ssl context\n");
         std::cout << "Error ssl context is null (check that openssl has been inited)" << std::endl;
 #endif
+        log_ssl_errors("Error ssl context is null for %s", hostname.c_str());
         return -1;
     }
 
@@ -244,26 +253,57 @@ int Socket::startSslClient(const std::string &certificate_path)
     }
 
     //load certs
+    ERR_clear_error();
     if (certificate_path.length()) {
         if (!SSL_CTX_load_verify_locations(ctx, NULL, certificate_path.c_str())) {
 #ifdef DGDEBUG
             std::cout << "couldnt load certificates" << std::endl;
 #endif
+            log_ssl_errors("couldnt load certificates from %s", certificate_path.c_str());
             //tidy up
             SSL_CTX_free(ctx);
+            ctx = NULL;
             return -2;
         }
     } else if (!SSL_CTX_set_default_verify_paths(ctx)) //use default if no certPpath given
     {
 #ifdef DGDEBUG
-        std::cout << "couldnt load certificates" << std::endl;
+        std::cout << "couldnt load default certificates" << std::endl;
 #endif
+            log_ssl_errors("couldnt load default certificates for %s", hostname.c_str());
         //tidy up
         SSL_CTX_free(ctx);
+        ctx = NULL;
         return -2;
     }
 
+    // add validation params
+    ERR_clear_error();
+    X509_VERIFY_PARAM *x509_param = X509_VERIFY_PARAM_new();
+    if (!x509_param) {
+        log_ssl_errors("couldnt add validation params for %s", hostname.c_str());
+        X509_VERIFY_PARAM_free(x509_param);
+        return -2;
+    }
+
+    ERR_clear_error();
+    if (!X509_VERIFY_PARAM_set_flags(x509_param, X509_V_FLAG_TRUSTED_FIRST)) {
+        log_ssl_errors("couldnt add validation params for %s", hostname.c_str());
+        X509_VERIFY_PARAM_free(x509_param);
+        return -2;
+    }
+
+    ERR_clear_error();
+    if (!SSL_CTX_set1_param(ctx, x509_param)) {
+        log_ssl_errors("couldnt add validation params for %s", hostname.c_str());
+        X509_VERIFY_PARAM_free(x509_param);
+        return -2;
+    }
+
+    X509_VERIFY_PARAM_free(x509_param);
+
     //hand socket over to ssl lib
+    ERR_clear_error();
     ssl = SSL_new(ctx);
     SSL_set_options(ssl, SSL_OP_ALL);
     SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY);
@@ -271,20 +311,25 @@ int Socket::startSslClient(const std::string &certificate_path)
 
     //fcntl(this->getFD() ,F_SETFL, O_NONBLOCK);
     SSL_set_fd(ssl, this->getFD());
+    SSL_set_tlsext_host_name(ssl, hostname.c_str());
 
     //make io non blocking as select wont tell us if we can do a read without blocking
     //BIO_set_nbio(SSL_get_rbio(ssl),1l);
     //BIO_set_nbio(SSL_get_wbio(ssl),1l);
+    ERR_clear_error();
     int rc = SSL_connect(ssl);
     if (rc < 0) {
-        ERR_print_errors_fp(stderr);
+        //ERR_print_errors_fp(stderr);
+        log_ssl_errors("ssl_connect failed to %s", hostname.c_str());
 //printerr(strerror(errno));
 #ifdef DGDEBUG
         std::cout << "ssl_connect failed with error " << SSL_get_error(ssl, rc) << std::endl;
 #endif
         // tidy up
         SSL_free(ssl);
+        ssl = NULL;
         SSL_CTX_free(ctx);
+        ctx = NULL;
         return -3;
     }
 
@@ -547,6 +592,9 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
         stopSsl();
     }
 
+    // set ssl to NULL
+    ssl = NULL;
+
     //setup the ssl server ctx
     ctx = SSL_CTX_new(SSLv23_server_method());
     if (ctx == NULL) {
@@ -584,28 +632,36 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
     }
 
     //setup the ssl session
+    ERR_clear_error();
     ssl = SSL_new(ctx);
     SSL_set_options(ssl, SSL_OP_ALL);
     SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY);
     SSL_set_accept_state(ssl);
 
+    ERR_clear_error();
     SSL_set_fd(ssl, this->getFD());
 
     //make io non blocking as select wont tell us if we can do a read without blocking
 
+    ERR_clear_error();
     if (SSL_accept(ssl) < 0) {
 #ifdef DGDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
         std::cout << "Error accepting ssl connection" << std::endl;
 #endif
+        log_ssl_errors("ssl_accept failed to client %s", "");
+        stopSsl();
         return -1;
     }
 
+    ERR_clear_error();
     if (SSL_do_handshake(ssl) < 0) {
 #ifdef DGDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
         std::cout << "Error doing ssl handshake" << std::endl;
 #endif
+        log_ssl_errors("ssl_handshake failed to client %s", "");
+        stopSsl();
         return -1;
     }
     isssl = true;
@@ -633,9 +689,10 @@ bool Socket::checkForInput()
     }
 
     //see if we can do an ssl read of 1 byte
-    char buf[1];
+//    char buf[1];
 
-    int rc = SSL_peek(ssl, buf, 1);
+    //int rc = SSL_peek(ssl, buf, 1);
+    int rc = SSL_pending(ssl);
 
     if (rc < 1) {
 #ifdef DGDEBUG
@@ -718,11 +775,11 @@ int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig,
     while (i < (size - 1)) {
         buffstart = 0;
         bufflen = 0;
-        try {
-            checkForInput(timeout, honour_reloadconfig);
-        } catch (std::exception &e) {
-            throw std::runtime_error(std::string("Can't read from socket: ") + strerror(errno)); // on error
-        }
+        //try {
+            //checkForInput(timeout, honour_reloadconfig);
+        //} catch (std::exception &e) {
+            //throw std::runtime_error(std::string("Can't read from socket: ") + strerror(errno)); // on error
+        //}
         bufflen = SSL_read(ssl, buffer, 1024);
 #ifdef DGDEBUG
 //std::cout << "read into buffer; bufflen: " << bufflen <<std::endl;
@@ -731,7 +788,8 @@ int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig,
             if (errno == EINTR && (honour_reloadconfig ? !reloadconfig : true)) {
                 continue;
             }
-            std::cout << "SSL_read failed with error " << SSL_get_error(ssl, bufflen) << std::endl;
+         //   std::cout << "SSL_read failed with error " << SSL_get_error(ssl, bufflen) << std::endl;
+            log_ssl_errors("ssl_read failed %s", "");
             throw std::runtime_error(std::string("Can't read from ssl socket")); //strerror(errno));  // on error
         }
         //if socket closed...
@@ -789,18 +847,20 @@ bool Socket::writeToSocket(const char *buff, int len, unsigned int flags, int ti
     int actuallysent = 0;
     int sent;
     while (actuallysent < len) {
-        if (check_first) {
-            try {
-                readyForOutput(timeout, honour_reloadconfig); // throws exception on error or timeout
-            } catch (std::exception &e) {
-                return false;
-            }
-        }
+        //if (check_first) {
+         //   try {
+               //readyForOutput(timeout, honour_reloadconfig); // throws exception on error or timeout
+            //} catch (std::exception &e) {
+                //return false;
+            //}
+        //}
+        ERR_clear_error();
         sent = SSL_write(ssl, buff + actuallysent, len - actuallysent);
         if (sent < 0) {
             if (errno == EINTR && (honour_reloadconfig ? !reloadconfig : true)) {
                 continue; // was interupted by signal so restart
             }
+            log_ssl_errors("ssl_write failed %s", "");
             return false;
         }
         if (sent == 0) {
@@ -843,6 +903,7 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
         } catch (std::exception &e) {
             return -1;
         }
+        ERR_clear_error();
         rc = SSL_read(ssl, buff, cnt);
 #ifdef DGDEBUG
         std::cout << "ssl read said: " << rc << std::endl;
@@ -852,6 +913,7 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
             if (errno == EINTR) {
                 continue;
             }
+            log_ssl_errors("ssl_read failed %s", "");
             return -1;
         }
         if (rc == 0) { // eof
@@ -889,21 +951,22 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
     }
 
     int rc;
-    if (check_first) {
-        try {
-            checkForInput(timeout, honour_reloadconfig);
-        } catch (std::exception &e) {
-            return -1;
-        }
-    }
+    //if (check_first) {
+        //try {
+            //checkForInput(timeout, honour_reloadconfig);
+        //} catch (std::exception &e) {
+            //return -1;
+        //}
+    //}
     while (true) {
-
+        ERR_clear_error();
         rc = SSL_read(ssl, buff, cnt);
 
         if (rc < 0) {
             if (errno == EINTR && (honour_reloadconfig ? !reloadconfig : true)) {
                 continue;
             }
+            log_ssl_errors("ssl_read failed %s", "");
         }
         break;
     }
